@@ -38,10 +38,11 @@ vector<AttachFile>::iterator
 AttachFileList::FindByFileId( int file_id )
 {
 	for( vector<AttachFile>::iterator ixfile = begin(); ixfile != end(); ixfile++ ) {
-		printf( "file_id  %d\n", file_id );
-		printf( "ixfile->FileId %d\n", ixfile->FileId() );
-		printf( "ixfile->FileName %s\n", ixfile->FileName().c_str() );
-		fflush( stdout );
+#ifdef DEBUG
+		printf( "file_id  %d\n", file_id );fflush(stdout);
+		printf( "ixfile->FileId %d\n", ixfile->FileId() );fflush(stdout);
+		printf( "ixfile->FileName %s\n", ixfile->FileName().c_str() );fflush(stdout);
+#endif
 		if ( file_id == ixfile->FileId() ) {
 			return ixfile;
 		}
@@ -55,10 +56,14 @@ AttachFileList::FindByFileId( int file_id )
  */
 AttachFile::AttachFile()
 {
-	printf("file_id before     == %d\n", file_id );
+#ifdef DEBUG
+	printf("file_id before     == %d\n", file_id );fflush(stdout);
+#endif
 	_FileId = file_id++;
-	printf("AttachFile::FileId == %d\n", FileId() );
-	printf("file_id after      == %d\n", file_id );
+#ifdef DEBUG
+	printf("AttachFile::FileId == %d\n", FileId() );fflush(stdout);
+	printf("file_id after      == %d\n", file_id );fflush(stdout);
+#endif
 }
 
 /**
@@ -78,7 +83,7 @@ AttachFile::GetLocalFileInfo()
 	}
 	setFileName( filename );
 	setLocation( location );
-	stat( FullPath().c_str(), &st );
+	lstat( FullPath().c_str(), &st );
 	setAttr( 0 );
 	if ( S_ISDIR( st.st_mode ) ) {
 		setAttr( IPMSG_FILE_DIR );
@@ -124,7 +129,9 @@ AttachFile::CreateDirFullPath( vector<string> dirstack )
 	for( vector<string>::iterator d = dirstack.begin(); d != dirstack.end(); d++ ){
 		if ( *d != "" ) {
 			retdir += *d + ( d->at(d->size() - 1) == '/' ? "" : "/" );
-			printf("retdir = %s\n", retdir.c_str());
+#ifdef DEBUG
+			printf("retdir = %s\n", retdir.c_str());fflush(stdout);
+#endif
 		}
 	}
 	return retdir;
