@@ -8,6 +8,7 @@
 #endif
 
 #include "IpMessenger.h"
+#include "IpMessengerImpl.h"
 #include "ipmsg.h"
 using namespace std;
 
@@ -17,32 +18,73 @@ using namespace std;
  * Υ᥽åɤϥåɥդǤʤ
  */
 bool
-RecievedMessage::DownloadFile( AttachFile &file, string saveFileNameFullPath, DownloadInfo& info )
+RecievedMessage::DownloadFile( AttachFile &file, string saveFileNameFullPath, DownloadInfo& info, FileNameConverter *conv, void *data )
+{
+#if defined(DEBUG)
+	printf("DownloadFile\n" );fflush(stdout);
+#endif
+	bool ret = true;
+	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
+	IpMessengerEvent *event = agent->GetEventObject();
+	FileNameConverter *converter = conv;
+	if ( conv == NULL ){
+		converter = new NullFileNameConverter();
+	}
+
+	info.setProcessing( true );
+	info.setFile( file );
+	info.setLocalFileName( saveFileNameFullPath );
+	if ( event == NULL ) {
+		ret = DownloadFilePrivate( NULL, file, saveFileNameFullPath, info, converter, data );
+	} else {
+		while( ret ) {
+			event->DownloadStart( *this, file, info, data );
+			if ( DownloadFilePrivate( event, file, saveFileNameFullPath, info, converter, data ) ) {
+				event->DownloadEnd( *this, file, info, data );
+				ret = true;
+				break;
+			} else {
+				ret = event->DownloadError( *this, file, info, data );
+			}
+		}
+	}
+	if ( conv == NULL ){
+		delete converter;
+	}
+	return ret;
+}
+
+/**
+ * ե
+ * Ф˥ե׵ѥåȤե롣
+ * Υ᥽åɤϥåɥդǤʤ
+ */
+bool
+RecievedMessage::DownloadFilePrivate( IpMessengerEvent *event, AttachFile &file, string saveFileNameFullPath, DownloadInfo& info, FileNameConverter *conv, void *data )
 {
 	struct sockaddr_in svr_addr;
 	int sock = socket( AF_INET, SOCK_STREAM, 0 );
 
 	svr_addr = MessagePacket().Addr();
 #if defined(DEBUG)
-printf("IP[%s]\n", inet_ntoa( svr_addr.sin_addr ) );
-fflush(stdout);
+char ipaddrbuf[100];
+printf("IP[%s]\n", inet_ntoa_r( svr_addr.sin_addr.s_addr, ipaddrbuf, sizeof( ipaddrbuf ) ) );fflush(stdout);
+printf("saveFileNameFullPath[%s]\n", saveFileNameFullPath.c_str() );fflush(stdout);
 #endif
 	if ( connect( sock, (struct sockaddr *)&svr_addr, sizeof( svr_addr ) ) != 0 ){
-#if defined(DEBUG)
-		printf("errno=[%s][%d]\n", strerror(errno), errno);
-#endif
-		herror("connect");
+		perror("connect");
+		info.setProcessing( false );
 		return false;
 	}
 
-	IpMessengerAgent *agent = IpMessengerAgent::GetInstance();
-
+	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
+	
 	char sendBuf[MAX_UDPBUF];
 	int sendBufLen;
 	char optBuf[MAX_UDPBUF];
 	int optBufLen = snprintf( optBuf, sizeof( optBuf ), "%lx:%x:0", MessagePacket().PacketNo(), file.FileId() );
 	sendBufLen = agent->CreateNewPacketBuffer( IPMSG_GETFILEDATA,
-												 agent->GetLoginName(), agent->GetHostName(),
+												 agent->LoginName(), agent->HostName(),
 												 optBuf, optBufLen,
 												 sendBuf, sizeof( sendBuf ) );
 	agent->SendTcpPacket( sock, sendBuf, sendBufLen );
@@ -50,6 +92,8 @@ fflush(stdout);
 	int fd = open( saveFileNameFullPath.c_str(), O_WRONLY | O_CREAT );
 	if ( fd < 0 ){
 		perror("open");
+		close( sock );
+		info.setProcessing( false );
 		return false;
 	}
 	fchmod( fd, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
@@ -58,14 +102,50 @@ fflush(stdout);
 	long long wroteSize = 0LL;
 	time_t startTime = time( NULL );
 	int read_len = recv( sock, readbuf, file.FileSize() - readSize > sizeof( readbuf ) ? sizeof( readbuf ) : file.FileSize() - readSize, 0 );
+	if ( read_len < 0 ) {
+		perror("recv");
+		close( sock );
+		close( fd );
+		info.setProcessing( false );
+		return false;
+	}
 	readSize += read_len;
 	while( read_len > 0 ){
 		int wrote_len = write( fd, readbuf, read_len );
+		if ( wrote_len < 0 ) {
+			perror("write");
+			close( sock );
+			close( fd );
+			info.setProcessing( false );
+			return false;
+		}
 		wroteSize += wrote_len;
+		info.setSize( readSize );
+		info.setTime( time( NULL ) - startTime );
+		if ( event != NULL ) {
+			event->DownloadProcessing( *this, file, info, data );
+		}
+#if defined(DEBUG)
+		printf( "read_len = %d\n", read_len );fflush(stdout);
+		printf( "readSize = %lld\n", readSize );fflush(stdout);
+		printf( "file.FileSize() = %lld\n", file.FileSize() );fflush(stdout);
+#endif
+		if ( file.FileSize() == readSize ) {
+			break;
+		}
 		read_len = recv( sock, readbuf, file.FileSize() - readSize > sizeof( readbuf ) ? sizeof( readbuf ) : file.FileSize() - readSize, 0 );
+		if ( read_len < 0 ) {
+			perror("recv");
+			close( sock );
+			close( fd );
+			info.setProcessing( false );
+			return false;
+		}
 		readSize += read_len;
 	}
-	printf("close");
+#if defined(DEBUG)
+	printf("close");fflush(stdout);
+#endif
 	close( fd );
 	close( sock );
 	struct utimbuf ubuf;
@@ -76,49 +156,106 @@ fflush(stdout);
 	info.setSize( readSize );
 	info.setTime( time( NULL ) - startTime );
 	info.setFileCount( 1L );
+	info.setProcessing( false );
+	if ( event != NULL ) {
+		event->DownloadProcessing( *this, file, info, data );
+	}
 	return true;
 }
 
-/**
- * ǥ쥯ȥ
- * Ф˥ǥ쥯ȥ׵ѥåȤǥ쥯ȥ롣
- * Υ᥽åɤϥåɥդǤʤ
- */
-bool
-RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBaseDir, DownloadInfo& info )
-{
-	NullFileNameConverter *codec = new NullFileNameConverter();
-	bool ret = DownloadDir( file, saveName, saveBaseDir, info, codec );
-	delete codec;
-	return ret;
-}
-
 /**
  * ǥ쥯ȥʥե̾Сץդˡ
  * Ф˥ǥ쥯ȥ׵ѥåȤǥ쥯ȥ롣
  * Υ᥽åɤϥåɥդǤʤ
  */
 bool
-RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBaseDir, DownloadInfo& info, FileNameConverter *conv )
+RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBaseDir, DownloadInfo& info, FileNameConverter *conv, void *data )
+{
+#if defined(DEBUG)
+	printf("DownloadDir\n" );fflush(stdout);
+#endif
+	bool ret = true;
+	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
+	IpMessengerEvent *event = agent->GetEventObject();
+	FileNameConverter *converter = conv;
+	if ( conv == NULL ){
+		converter = new NullFileNameConverter();
+	}
+	
+	info.setFile( file );
+	info.setLocalFileName( GetSaveDir( saveName, saveBaseDir ) );
+	info.setProcessing( true );
+	if ( event == NULL ) {
+		ret = DownloadDirPrivate( NULL, file, saveName, saveBaseDir, info, converter, data );
+	} else {
+		while( ret ) {
+			event->DownloadStart( *this, file, info, data );
+			if ( DownloadDirPrivate( event, file, saveName, saveBaseDir, info, converter, data ) ) {
+				event->DownloadEnd( *this, file, info, data );
+				ret = true;
+				break;
+			} else {
+				ret = event->DownloadError( *this, file, info, data );
+			}
+		}
+	}
+	if ( conv == NULL ){
+		delete converter;
+	}
+	return ret;
+}
+
+string
+RecievedMessage::GetFormalDir( string dirName )
+{
+	if ( dirName.at( dirName.length() - 1 ) != '/' ) {
+		return dirName + "/";
+	}
+	return dirName;
+}
+
+string
+RecievedMessage::GetSaveDir( string saveName, string saveBaseDir )
+{
+	return GetFormalDir( saveBaseDir ) + saveName + "/";
+}
+
+/**
+ * ǥ쥯ȥʥե̾Сץդˡ
+ * Ф˥ǥ쥯ȥ׵ѥåȤǥ쥯ȥ롣
+ * Υ᥽åɤϥåɥդǤʤ
+ */
+bool
+RecievedMessage::DownloadDirPrivate( IpMessengerEvent *event, AttachFile &file, string saveName, string saveBaseDir, DownloadInfo& info, FileNameConverter *conv, void *data )
 {
 	if ( conv == NULL ) {
+		info.setProcessing( false );
 		return false;
 	}
 	struct stat st;
-	string saveBaseDirFormal = saveBaseDir;
-	if ( saveBaseDirFormal.at( saveBaseDirFormal.length() - 1 ) != '/' ) {
-		saveBaseDirFormal = saveBaseDir + "/";
-	}
-	string saveDir = saveBaseDirFormal + saveName + "/";
+	string saveBaseDirFormal = GetFormalDir( saveBaseDir );
+	string saveDir = GetSaveDir( saveName, saveBaseDir );
 
+#if defined(DEBUG)
+printf("saveName[%s]\n", saveName.c_str() );fflush(stdout);
+printf("saveBaseDir[%s]\n", saveBaseDir.c_str() );fflush(stdout);
+printf("saveDir[%s]\n", saveDir.c_str() );fflush(stdout);
+printf("saveBaseDirFormal[%s]\n", saveBaseDirFormal.c_str() );fflush(stdout);
+#endif
 	if ( stat( saveBaseDir.c_str(), &st ) != 0 ) {
 		perror("stat");
-		printf("saveBaseDir == [%s]\n", saveBaseDir.c_str());
+#if defined(DEBUG)
+		printf("saveBaseDir == [%s]\n", saveBaseDir.c_str());fflush(stdout);
+#endif
+		info.setProcessing( false );
 		return false;
 	}
 	if ( mkdir( saveDir.c_str(), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) != 0 ) {
-		perror("mkdir");
-		printf("saveDir == [%s]\n", saveDir.c_str());
+		perror("mkdir(1)");
+#if defined(DEBUG)
+		printf("saveDir == [%s]\n", saveDir.c_str());fflush(stdout);
+#endif
+		info.setProcessing( false );
 		return false;
 	}
 
@@ -131,17 +268,18 @@ RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBase
 	svr_addr = MessagePacket().Addr();
 	if ( connect( sock, (struct sockaddr *)&svr_addr, sizeof( svr_addr ) ) != 0 ){
 		perror("connect");
+		info.setProcessing( false );
 		return false;
 	}
 
-	IpMessengerAgent *agent = IpMessengerAgent::GetInstance();
+	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
 
 	char sendBuf[MAX_UDPBUF];
 	int sendBufLen;
 	char optBuf[MAX_UDPBUF];
 	int optBufLen = snprintf( optBuf, sizeof( optBuf ), "%lx:%x", MessagePacket().PacketNo(), file.FileId() );
 	sendBufLen = agent->CreateNewPacketBuffer( IPMSG_GETDIRFILES,
-												 agent->GetLoginName(), agent->GetHostName(),
+												 agent->LoginName(), agent->HostName(),
 												 optBuf, optBufLen,
 												 sendBuf, sizeof( sendBuf ) );
 	agent->SendTcpPacket( sock, sendBuf, sendBufLen );
@@ -159,13 +297,12 @@ RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBase
 	time_t startTime = time( NULL );
 #define HEADER_LENGTH_LEN	5
 #if defined(DEBUG)
-	printf("start DIR [%s]\n", AttachFile::CreateDirFullPath( dir ).c_str() );
+	printf("start DIR [%s]\n", AttachFile::CreateDirFullPath( dir ).c_str() );fflush(stdout);
 #endif
 	while( !isEob ){
 #if defined(DEBUG)
-		fflush(stdout);
-		printf("saveBaseDirFormal[%s]\n", saveBaseDirFormal.c_str() );
-		printf("AttachFile::CreateDirFullPath( dir )[%s]\n", AttachFile::CreateDirFullPath( dir ).c_str() );
+		printf("saveBaseDirFormal[%s]\n", saveBaseDirFormal.c_str() );fflush(stdout);
+		printf("AttachFile::CreateDirFullPath( dir )[%s]\n", AttachFile::CreateDirFullPath( dir ).c_str() );fflush(stdout);
 #endif
 		if ( saveBaseDirFormal == AttachFile::CreateDirFullPath( dir ) ) {
 			isEob = true;
@@ -179,7 +316,7 @@ RecievedMessage::DownloadDir( AttachFile &file, string saveName, string saveBase
 			break;
 		}
 #if defined(DEBUG)
-printf("HEADER=%d\n", read_len );
+printf("HEADER=%d\n", read_len );fflush(stdout);
 IpMsgPrintBuf( "DownloadDir:readbuf", readbuf, read_len );
 #endif
 		if ( read_len < HEADER_LENGTH_LEN ) {
@@ -192,7 +329,7 @@ IpMsgPrintBuf( "DownloadDir:readbuf", readbuf, read_len );
 		memset( readbuf, 0, sizeof( readbuf ) );
 		read_len = recv( sock, readbuf, header_len - HEADER_LENGTH_LEN, 0 );
 #if defined(DEBUG)
-printf("NEXT RECV=%d\n", header_len - HEADER_LENGTH_LEN);
+printf("NEXT RECV=%d\n", header_len - HEADER_LENGTH_LEN);fflush(stdout);
 IpMsgPrintBuf( "DownloadDir:readbuf2", readbuf, read_len );
 #endif
 		if ( read_len < 0 ) {
@@ -213,17 +350,17 @@ IpMsgPrintBuf( "DownloadDir:readbuf2", readbuf, read_len );
 			int fd = open( FullPath.c_str(), O_WRONLY | O_CREAT );
 			if ( fd < 0 ){
 				perror("open");
+				info.setProcessing( false );
 				return false;
 			}
 			fchmod( fd, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
 			memset( readbuf, 0, sizeof( readbuf ) );
 #if defined(DEBUG)
-printf("FileSize=%lld\n", f.FileSize() );
-fflush(stdout);
+printf("FileSize=%lld\n", f.FileSize() );fflush(stdout);
 #endif
 			int read_len = recv( sock, readbuf, f.FileSize() - readSize > sizeof( readbuf ) ? sizeof( readbuf ) : f.FileSize() - readSize, 0 );
 #if defined(DEBUG)
-//IpMsgPrintBuf( "DownloadDir:readbuf3", readbuf, read_len );
+IpMsgPrintBuf( "DownloadDir:readbuf3", readbuf, read_len );
 #endif
 			readSize += read_len;
 			while( read_len > 0 ){
@@ -232,8 +369,20 @@ fflush(stdout);
 					isEob = true;
 					break;
 				}
-
-				wroteSize += write( fd, readbuf, read_len );
+				int wrote_len = write( fd, readbuf, read_len );
+				if ( wrote_len < 0 ) {
+					perror("write");
+					close( sock );
+					close( fd );
+					info.setProcessing( false );
+					return false;
+				}
+				wroteSize += wrote_len;
+				info.setSize( totalReadSize );
+				info.setTime( time( NULL ) - startTime );
+				if ( event != NULL ) {
+					event->DownloadProcessing( *this, file, info, data );
+				}
 				memset( readbuf, 0, sizeof( readbuf ) );
 				read_len = recv( sock, readbuf, f.FileSize() - readSize > sizeof( readbuf ) ? sizeof( readbuf ) : f.FileSize() - readSize, 0 );
 #if defined(DEBUG)
@@ -250,7 +399,7 @@ fflush(stdout);
 			info.setTime( time( NULL ) - startTime );
 			info.setFileCount( ++totalFileCount );
 #if defined(DEBUG)
-			printf("read=%lld,wrote=%lld\n", readSize, wroteSize);
+			printf("read=%lld,wrote=%lld\n", readSize, wroteSize);fflush(stdout);
 #endif
 		} else if ( GET_FILETYPE( f.Attr() ) == IPMSG_FILE_DIR ) {
 			if ( isTopDir ) {
@@ -260,19 +409,21 @@ fflush(stdout);
 			dir.push_back( f.FileName().c_str() );
 			string FullPath = f.CreateDirFullPath( dir );
 			if ( mkdir( FullPath.c_str(), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) != 0 ) {
-				perror("mkdir");
+				perror("mkdir(2)");
 				isEob = true;
 				break;
 			}
 			info.setTime( time( NULL ) - startTime );
 			info.setFileCount( ++totalFileCount );
+			if ( event != NULL ) {
+				event->DownloadProcessing( *this, file, info, data );
+			}
 		} else if ( GET_FILETYPE( f.Attr() ) == IPMSG_FILE_RETPARENT ) {
 			dir.pop_back();
 		}
 	}
 #if defined(DEBUG)
-	printf("close socket");
-	fflush(stdout);
+	printf("close socket");fflush(stdout);
 #endif
 	close( sock );
 	file.setIsDownloading( false );
@@ -280,5 +431,9 @@ fflush(stdout);
 	info.setSize( totalReadSize );
 	info.setTime( time( NULL ) - startTime );
 	info.setFileCount( totalFileCount );
+	info.setProcessing( false );
+	if ( event != NULL ) {
+		event->DownloadProcessing( *this, file, info, data );
+	}
 	return true;
 }
